package com.htfp.jni.uav98.api;


import com.htfp.jni.uav98.bean.Uav98FlightDataFrame;
import com.htfp.jni.utils.LoadLibUtil;

import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;
import java.util.function.Function;


/**
 * @author Dong Jw
 * @creatTime 2024-01-03 10:56
 */
public class Uav98FlightDataApi {
    private static volatile Uav98FlightDataApi instance;
    private static final ConcurrentMap<Byte, Function<byte[], Uav98FlightDataFrame>> DECODE_FUNCTIONS = new ConcurrentHashMap<>();

    static {
        DECODE_FUNCTIONS.put((byte) 0x81, Uav98FlightDataJNI::decode81_84);
        DECODE_FUNCTIONS.put((byte) 0x85, Uav98FlightDataJNI::decode85_89);
        DECODE_FUNCTIONS.put((byte) 0x8B, Uav98FlightDataJNI::decode8B_8E);
        DECODE_FUNCTIONS.put((byte) 0x8F, (bytes) -> {
            byte subStartByte = bytes[44];
            switch (subStartByte) {
                case (byte) 0x91:
                    return Uav98FlightDataJNI.decode8F_93(bytes);
                case (byte) 0x94:
                    return Uav98FlightDataJNI.decode8F_96(bytes);
                default:
                    throw new IllegalArgumentException("Invalid byte value: " + subStartByte);
            }
        });
    }

    private Uav98FlightDataApi() {
    }

    public static Uav98FlightDataApi getInstance() {
        if (instance == null) {
            synchronized (Uav98FlightDataApi.class) {
                if (instance == null) {
                    instance = new Uav98FlightDataApi();
                    LoadLibUtil.loadLib("libDecode98");
                }
            }
        }
        return instance;
    }

    public Uav98FlightDataFrame decode(byte[] bytes) throws IllegalArgumentException {
        byte startByte = bytes[12];
        if (!DECODE_FUNCTIONS.containsKey(startByte)) {
            throw new IllegalArgumentException("Invalid byte value: " + startByte);
        }
        return DECODE_FUNCTIONS.get(startByte).apply(bytes);
    }
}
