package com.htfp.jni.uav981A.api;


import com.htfp.jni.uav981A.bean.Uav981AFlightDataFrame;
import com.htfp.jni.utils.LoadLibUtil;

import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;
import java.util.function.Function;

/**
 * @author Dong Jw
 * @creatTime 2024-01-03 10:56
 */
public class Uav981AFlightDataApi {
    private static volatile Uav981AFlightDataApi instance;
    private static final ConcurrentMap<Byte, Function<byte[], Uav981AFlightDataFrame>> DECODE_FUNCTIONS = new ConcurrentHashMap<>();

    static {
        DECODE_FUNCTIONS.put((byte) 0xA0, Uav981AFlightDataJNI::decodeA0);
        DECODE_FUNCTIONS.put((byte) 0xA1, Uav981AFlightDataJNI::decodeA1);
        DECODE_FUNCTIONS.put((byte) 0xA2, Uav981AFlightDataJNI::decodeA2);
        DECODE_FUNCTIONS.put((byte) 0xA3, Uav981AFlightDataJNI::decodeA3);
        DECODE_FUNCTIONS.put((byte) 0xA4, Uav981AFlightDataJNI::decodeA4);
        DECODE_FUNCTIONS.put((byte) 0xA5, Uav981AFlightDataJNI::decodeA5);
        DECODE_FUNCTIONS.put((byte) 0xA6, Uav981AFlightDataJNI::decodeA6);
        DECODE_FUNCTIONS.put((byte) 0xA7, Uav981AFlightDataJNI::decodeA7);
        DECODE_FUNCTIONS.put((byte) 0xA8, Uav981AFlightDataJNI::decodeA8);
        DECODE_FUNCTIONS.put((byte) 0xA9, Uav981AFlightDataJNI::decodeA9);
        DECODE_FUNCTIONS.put((byte) 0xAA, Uav981AFlightDataJNI::decodeAA);
        DECODE_FUNCTIONS.put((byte) 0xAB, Uav981AFlightDataJNI::decodeAB);
        DECODE_FUNCTIONS.put((byte) 0xAC, Uav981AFlightDataJNI::decodeAC);
        DECODE_FUNCTIONS.put((byte) 0xAD, Uav981AFlightDataJNI::decodeAD);
        DECODE_FUNCTIONS.put((byte) 0xAE, Uav981AFlightDataJNI::decodeAE);
    }

    private Uav981AFlightDataApi() {
    }

    public static Uav981AFlightDataApi getInstance() {
        if (instance == null) {
            synchronized (Uav981AFlightDataApi.class) {
                if (instance == null) {
                    instance = new Uav981AFlightDataApi();
                    LoadLibUtil.loadLib("libDecode981A");
                }
            }
        }
        return instance;
    }

    public Uav981AFlightDataFrame decode(byte[] bytes) throws IllegalArgumentException {
        if (!DECODE_FUNCTIONS.containsKey(bytes[2])) {
            throw new IllegalArgumentException("Invalid byte value: " + bytes[2]);
        }
        return DECODE_FUNCTIONS.get(bytes[2]).apply(bytes);
    }
}
