package com.htfp.jni.uav981C.api;

import com.htfp.jni.uav981C.bean.Uav981CFlightDataFrame;
import com.htfp.jni.utils.LoadLibUtil;

import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;
import java.util.function.Function;

/**
 * @author Dong Jw
 * @creatTime 2024-01-09 09:10
 */
public class Uav981CFlightDataApi {
    private static volatile Uav981CFlightDataApi instance;
    private static final ConcurrentMap<Byte, Function<byte[], Uav981CFlightDataFrame>> DECODE_FUNCTIONS = new ConcurrentHashMap<>();

    static {
        DECODE_FUNCTIONS.put((byte) 0xC0, Uav981CFlightDataJNI::decodeC0);
        DECODE_FUNCTIONS.put((byte) 0xC1, Uav981CFlightDataJNI::decodeC1);
        DECODE_FUNCTIONS.put((byte) 0xC2, Uav981CFlightDataJNI::decodeC2);
        DECODE_FUNCTIONS.put((byte) 0xC3, Uav981CFlightDataJNI::decodeC3);
        DECODE_FUNCTIONS.put((byte) 0xC4, Uav981CFlightDataJNI::decodeC4);
        DECODE_FUNCTIONS.put((byte) 0xC5, Uav981CFlightDataJNI::decodeC5);
        DECODE_FUNCTIONS.put((byte) 0xC6, Uav981CFlightDataJNI::decodeC6);
        DECODE_FUNCTIONS.put((byte) 0xC7, Uav981CFlightDataJNI::decodeC7);
        DECODE_FUNCTIONS.put((byte) 0xC8, Uav981CFlightDataJNI::decodeC8);
        DECODE_FUNCTIONS.put((byte) 0xC9, Uav981CFlightDataJNI::decodeC9);
        DECODE_FUNCTIONS.put((byte) 0xCA, Uav981CFlightDataJNI::decodeCA);
        DECODE_FUNCTIONS.put((byte) 0xCB, Uav981CFlightDataJNI::decodeCB);
        DECODE_FUNCTIONS.put((byte) 0xCC, Uav981CFlightDataJNI::decodeCC);
        DECODE_FUNCTIONS.put((byte) 0xCD, Uav981CFlightDataJNI::decodeCD);
        DECODE_FUNCTIONS.put((byte) 0xCE, Uav981CFlightDataJNI::decodeCE);
        DECODE_FUNCTIONS.put((byte) 0xCF, Uav981CFlightDataJNI::decodeCF);
        DECODE_FUNCTIONS.put((byte) 0xD0, Uav981CFlightDataJNI::decodeD0);
        DECODE_FUNCTIONS.put((byte) 0xD1, Uav981CFlightDataJNI::decodeD1);
        DECODE_FUNCTIONS.put((byte) 0xD2, Uav981CFlightDataJNI::decodeD2);
    }

    private Uav981CFlightDataApi() {
    }

    public static Uav981CFlightDataApi getInstance() {
        if (instance == null) {
            synchronized (Uav981CFlightDataApi.class) {
                if (instance == null) {
                    instance = new Uav981CFlightDataApi();
                    LoadLibUtil.loadLib("libDecode981C");
                }
            }
        }
        return instance;
    }

    public Uav981CFlightDataFrame decode(byte[] bytes) {
        if (!DECODE_FUNCTIONS.containsKey(bytes[2])) {
            throw new IllegalArgumentException("Invalid byte value: " + bytes[2]);
        }
        return DECODE_FUNCTIONS.get(bytes[2]).apply(bytes);
    }
}
