package com.htfp.jni.uav985.api;

import com.htfp.jni.uav985.bean.Uav985FlightDataFrame;
import com.htfp.jni.utils.LoadLibUtil;

import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;
import java.util.function.Function;

/**
 * @author Dong Jw
 * @creatTime 2024-01-11 11:52
 */
public class Uav985FlightDataApi {
    private static volatile Uav985FlightDataApi instance;
    private static final ConcurrentMap<Byte, Function<byte[], Uav985FlightDataFrame>> DECODE_FUNCTIONS = new ConcurrentHashMap<>();

    static {
        DECODE_FUNCTIONS.put((byte) 0x81, Uav985FlightDataJNI::decode81_84);
        DECODE_FUNCTIONS.put((byte) 0x85, Uav985FlightDataJNI::decode85_89);
        DECODE_FUNCTIONS.put((byte) 0x8B, Uav985FlightDataJNI::decode8B_8E);
        DECODE_FUNCTIONS.put((byte) 0x8F, (bytes) -> {
            byte subStartByte = bytes[44];
            switch (subStartByte) {
                case (byte) 0x91:
                    return Uav985FlightDataJNI.decode8F_93(bytes);
                case (byte) 0x94:
                    return Uav985FlightDataJNI.decode8F_96(bytes);
                default:
                    throw new IllegalArgumentException("Invalid byte value: " + subStartByte);
            }
        });
    }

    private Uav985FlightDataApi() {
    }

    public static Uav985FlightDataApi getInstance() {
        if (instance == null) {
            synchronized (Uav985FlightDataApi.class) {
                if (instance == null) {
                    instance = new Uav985FlightDataApi();
                    LoadLibUtil.loadLib("libDecode985");
                }
            }
        }
        return instance;
    }

    public Uav985FlightDataFrame decode(byte[] bytes) throws IllegalArgumentException {
        byte startByte = bytes[12];
        if (!DECODE_FUNCTIONS.containsKey(startByte)) {
            throw new IllegalArgumentException("Invalid byte value: " + startByte);
        }
        return DECODE_FUNCTIONS.get(startByte).apply(bytes);
    }
}
