package com.htfp.jni.utils;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Objects;
import java.util.concurrent.locks.ReentrantLock;

/**
 * @author Dong Jw
 * @creatTime 2024-01-09 14:12
 */
public final class LoadLibUtil {
    private static final String BIN_LIB = "/";
    private  static final ReentrantLock lock = new ReentrantLock();

    private LoadLibUtil() {
    }

    public static void loadLib(String libName) {
        String systemType = System.getProperty("os.name");
        String libExtension = (systemType.toLowerCase().contains("win")) ? ".dll" : ".so";
        String libFullName = libName + libExtension;
        String nativeTempDir = System.getProperty("java.io.tmpdir");
        File extractedLibFile = new File(nativeTempDir + File.separator + libFullName);

        if (!extractedLibFile.exists()) {
            lock.lock();
            try (InputStream in = LoadLibUtil.class.getResourceAsStream(BIN_LIB + libFullName);
                 BufferedInputStream reader = new BufferedInputStream(Objects.requireNonNull(in));
                 FileOutputStream writer = new FileOutputStream(extractedLibFile)) {

                byte[] buffer = new byte[1024];
                while (true) {
                    int bytesRead = reader.read(buffer);
                    if (bytesRead == -1) {
                        break;
                    }
                    writer.write(buffer, 0, bytesRead);
                }
            } catch (IOException e) {
                e.printStackTrace();
            } finally {
                lock.unlock();
            }
        }
        System.load(extractedLibFile.toString());
    }
}
